@@ -1,4 +1,4 @@
-;;; test-org-macro.el --- Tests for org-macro.el
+;;; test-org-macro.el --- Tests for org-macro.el  -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2013, 2014, 2019  Nicolas Goaziou
 
@@ -15,13 +15,28 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Code:
 
 
 ;;; Macros
 
+(ert-deftest test-org-macro/initialize-templates ()
+  "Test `org-macro-initialize-templates'."
+  ;; No code is executed during loading of Org mode files."
+  (should
+   (org-test-with-temp-text
+       "#+MACRO: title (eval (eval-and-compile (error \"CVE-2024-30202\")))"
+     (progn
+       (org-macro-initialize-templates)
+       t)))
+  (org-test-with-temp-text
+      "#+MACRO: title (eval (eval-and-compile (error \"CVE-2024-30202\")))"
+    (progn
+      (org-mode)
+      t)))
+
 (ert-deftest test-org/macro-replace-all ()
   "Test `org-macro-replace-all' specifications."
   ;; Standard test.
@@ -314,6 +329,15 @@
 	    (org-macro-replace-all org-macro-templates)
 	    (buffer-substring-no-properties
 	     (line-beginning-position) (point-max)))))
+  ;; Return AUTHOR keyword value.
+  (should
+   (equal "author 1 author 2"
+	  (org-test-with-temp-text
+              "#+author: author 1\n#+author: author 2\n<point>{{{author}}}"
+	    (org-macro-initialize-templates)
+	    (org-macro-replace-all org-macro-templates)
+	    (buffer-substring-no-properties
+	     (line-beginning-position) (point-max)))))
   ;; When AUTHOR keyword is missing, return the empty string.
   (should
    (equal ""
